/* myMusic
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2004-2006, Toni Thomsson <toni@tonjac.org> 
*/
 
// Implementation of class Cmp3Streamer

#include <zz_http.h>

#include "mp3_streamer.h"
#include "mp3_http_requester.h"
#include "mp3_playlist.h"
#include "mp3_source_config.h"
#include <shoutcc.h>

CzzWinThreadMutex Cmp3Streamer::m_Mutex;

// Construction
Cmp3Streamer::Cmp3Streamer( Cmp3PlayListHandler* ph, const CzzString& usr )
 : CzzWinThread(), m_PlayListHandler( ph ), m_Log( Cmp3SourceConfig::appName().c_str() ), m_User(usr)
{

}

// Destruction
Cmp3Streamer::~Cmp3Streamer()
{

}


void Cmp3Streamer::OnStart()
{
    CzzString mount;
	Cmp3HttpRequester server;
	Shout s;
	ShoutMetadata md;
	static unsigned char buff[4096]; 
	long read = 0;
	long total = 0;
	FILE* f;
	int size = 0;
	bool http = true;

	try
	{
       	mount = CzzString::Format( "%s/%s", Cmp3SourceConfig::getSourceMount().c_str(), m_User.c_str() );
    	m_Log.Print( _INFOPARAMS, "Starting stream thread for mount %s...", mount.c_str() );	

		if( Cmp3SourceConfig::getServerProtocol() == "http" )
			http = true;
		else
			http = false;

		s.set_user( Cmp3SourceConfig::getSourceUser().c_str() );
		s.set_password( Cmp3SourceConfig::getSourcePwd().c_str() );
		
		if( http )
			s.set_mount( mount.c_str() );

		s.set_port( atoi( Cmp3SourceConfig::getServerPort().c_str() ) );
		s.set_host( Cmp3SourceConfig::getServerHost().c_str() );
		s.set_format( SHOUT_FORMAT_MP3 );

		if( http )
			s.set_protocol( SHOUT_PROTOCOL_HTTP );
		else
			s.set_protocol( SHOUT_PROTOCOL_ICY );

		s.set_name( CzzString::Format( "%s - %s", Cmp3SourceConfig::getSourceName().c_str(), m_User.c_str() ).c_str() );
		s.set_audio_info( SHOUT_AI_BITRATE, Cmp3SourceConfig::getSourceBitrate().c_str() );
		
		s.open(); 

		f = m_PlayListHandler->getPlayList(m_User)->getNextStreamHandle();
		//s.set_audio_info( SHOUT_AI_BITRATE, CzzString::Format( "%d", m_PlayListHandler->getPlayList(m_User)->getBitRate() ).c_str() );
		
		bool newsong = true;

		while( !m_StopRequest ) 
		{
			if( f )
			{
				read = fread( buff, 1, sizeof(buff), f );
				total = total + read;
			}

			if (read > 0) 
			{
				s.send(buff, read);
				s.sync();

				if( f && newsong )
				{
					newsong = false;

					if( http )
						server.updateSong( m_User, m_PlayListHandler->getPlayList(m_User)->nowPlaying().c_str() );
					else
					{
						md.add( "song", m_PlayListHandler->getPlayList(m_User)->nowPlaying().c_str() );
						s.set_metadata( &md );
					}
				}
			}
			else 
			{
				f = m_PlayListHandler->getPlayList(m_User)->getNextStreamHandle();
				//s.set_audio_info( SHOUT_AI_BITRATE, CzzString::Format( "%d", m_PlayListHandler->getPlayList(m_User)->getBitRate() ).c_str() );
				newsong = true;
			}
		}
	}
	catch( ShoutErr& serr )
	{
		m_Log.Print( _WARNINGPARAMS, "Couldn't stream %s, exiting stream thread", serr.err.c_str() );	
	}
	catch( CzzException& e )
	{
		e;
		m_Log.Print( _WARNINGPARAMS, "Couldn't stream (see prev. error), exiting stream thread");		
	}
	catch( ... )
	{
		m_Log.Print( _WARNINGPARAMS, "Couldn't stream (undefined reason), exiting stream thread");
	}

	fclose(f); 
	s.close();

	m_Log.Print( _INFOPARAMS, "Exiting stream thread for mount %s...", mount.c_str() );	
	m_Running = false;
}

