/* myMusic
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2004-2006, Toni Thomsson <toni@tonjac.org> 
*/

#pragma warning(disable: 4251)

#include <zz_socket.h>
#include "mp3_util.h"
#include "zz_log_exception.h"
#include "mp3_api.h"

#ifdef _WIN32
	static HINSTANCE hInstLib = (HINSTANCE)0;  // DLL Instance Handle
#endif

static Cmp3Util* g_API = 0;

#define DO_CATCH    catch( CzzException& e )\
	                {\
		                result = e.GetCode();\
                        g_API->setError( e.GetReason(), e.GetCode() );\
	                }\
	                catch( ... )\
	                {\
		                CzzLogException e( _LAF, MP3_UNDEFINED_ERROR, "Undefined error. Perhaps invalid input?" );\
		                result = e.GetCode();\
                        g_API->setError( e.GetReason(), e.GetCode() );\
	                }\

#define CHECK_API( m ) if( !g_API )\
                                throw CzzLogException( _LAF, MP3_ATTACH_FAILED, "Attach failed" );

// Get global IP adress  
extern "C"
MP3API mp3_getGlobalIPAdress( char* buff, short size )
{
	short result = 0;

	try
	{
		CzzString ip;

		CHECK_API( g_API )

		ip = g_API->getGlobalIPAdress( );

		if( ip.size() > size )
			throw CzzLogException( _LAF, MP3_BUFF_TO_SMALL, CzzString::Format( "Buffer to small, %d used %d required", size, ip.size() ));

		strcpy( buff, ip.c_str() );

	}
	
    DO_CATCH

	return result;
}

// Get last occured error
extern "C"
MP3API mp3_getLastError( short* code, char* buff, short size )
{
	short result = 0;

	try
	{
		CzzString msg;
		
        CHECK_API( g_API )

		msg = g_API->getLastError( code );

		if( msg.size() > size )
			throw CzzLogException( _LAF, MP3_BUFF_TO_SMALL, CzzString::Format( "Buffer to small, %d used %d required", size, msg.size() ));

		strcpy( buff, msg.c_str() );
	}

    DO_CATCH

	return result;
}

// DLL main
__declspec(dllexport) BOOL WINAPI DllMain( HINSTANCE hInst, DWORD fdwReason, LPVOID lpvReserved )
{
    try
    {
	    switch( fdwReason ) 
	    {
		    case DLL_PROCESS_ATTACH:
			    
                if( !hInstLib ) 
			    {    
   				    // used to flag that the DLL has been initialized
				    hInstLib = hInst; 
	                CzzSocket::Init();

                    g_API = new Cmp3Util();
                
			    }
			    
                break;

		    case DLL_PROCESS_DETACH:

                if( g_API )
                    delete g_API;

                CzzSocket::Exit();
			    break;
       }
    }
    catch( ... )
    {
        return FALSE;
    }

    return TRUE;
}

