/* myMusic
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2004-2006, Toni Thomsson <toni@tonjac.org> 
*/

// Implementation of class Cmp3InfoData

#include "mp3_info_data.h"
#include "mp3_svc.h"
#include "mp3_config.h"

#include <time.h>
#include <sys/timeb.h>
#include <sys/types.h> 
#include <id3/tag.h>
#include <zz_file.h>
#include <winver.h>

// Construction
Cmp3InfoData::Cmp3InfoData( void )
 : CzzDataService( "getInfoData" )
{

}

// Destruction
Cmp3InfoData::~Cmp3InfoData()
{

}

CzzDataServiceReply* Cmp3InfoData::execute( void* session, CzzParameterMap* req )
{

	CzzDataService::execute( session, req );

	CzzDataServiceReply* reply = new CzzDataServiceReply();

	CzzString path = Cmp3Config::getDataDir();
	_finddata_t finfo;
	long handle = _findfirst( (CzzString(path) + "\\*.*").c_str(), &finfo );
	int error = 0;
	int albums = 0;
	int artists = 0;
	int songs = 0;
	Cmp3List latest_added;

	// Read the dir
	if( (handle!= ENOENT)  && (handle != EINVAL) && (handle != -1) )
	{
		while( !error )
		{
			// We're not interested in the . and .. 
			if( (stricmp( finfo.name, "." ) != 0) && (stricmp( finfo.name, ".." ) != 0) )
			{
				if( (finfo.attrib & _A_SUBDIR) && !(finfo.attrib & _A_SYSTEM) )
				{
					countAlbumsAndSongs( finfo.name, (CzzString(path) + "\\" + CzzString(finfo.name)).c_str(), &albums, &songs, &latest_added );
					artists++;
				}

			}
			// Next file
			error = _findnext( handle, &finfo );
		}
	}

	_findclose( handle );

	char str[10];

	reply->addField( MP3_FLD_NO_ARTISTS, itoa(artists,str,10) );
	reply->addField( MP3_FLD_NO_ALBUMS, itoa(albums,str,10) );
	reply->addField( MP3_FLD_NO_SONGS, itoa(songs,str,10) );

	return reply;
}

void Cmp3InfoData::countAlbumsAndSongs( const char* artist, const char* path, int* albums, int* songs, Cmp3List* list )
{
	_finddata_t finfo;
	long handle = _findfirst( (CzzString(path) + "\\*.*").c_str(), &finfo );
	int error = 0;
	CzzString cgidir = Cmp3Config::getCGIDir();
	CzzString url;

	// Read the dir
	if( (handle!= ENOENT)  && (handle != EINVAL) && (handle != -1) )
	{
		while( !error )
		{
			// We're not interested in the . and .. 
			if( (stricmp( finfo.name, "." ) != 0) && (stricmp( finfo.name, ".." ) != 0) )
			{
				if( (finfo.attrib & _A_SUBDIR) && !(finfo.attrib & _A_SYSTEM) )
				{
					countSongs( (CzzString(path) + "\\" + CzzString(finfo.name)).c_str(), songs, list );
					(*albums)++;
					
					url = CzzString::Format( "<a href='%s/mp3_gui.exe?svc=1&name=%s'>%s</a> - <a href='%s/mp3_gui.exe?svc=2&name=%s&album=%s'>%s</a>",
							cgidir.c_str(), urlEncode(artist).c_str(), artist, cgidir.c_str(), urlEncode(artist).c_str(), urlEncode(finfo.name).c_str(), finfo.name );

					list->add( url.c_str(), finfo.time_write );
				}

			}
			// Next file
			error = _findnext( handle, &finfo );
		}
	}

	_findclose( handle );
}

void Cmp3InfoData::countSongs( const char* path, int* songs, Cmp3List* list )
{
	_finddata_t finfo;
	long handle = _findfirst( (CzzString(path) + "\\*.mp3").c_str(), &finfo );
	int error = 0;
	
	// Read the dir
	if( (handle!= ENOENT)  && (handle != EINVAL) && (handle != -1) )
	{
		while( !error )
		{
			// We're not interested in the . and .. 
			if( (stricmp( finfo.name, "." ) != 0) && (stricmp( finfo.name, ".." ) != 0) )
			{
				if( !(finfo.attrib & _A_SUBDIR) && !(finfo.attrib & _A_SYSTEM) )
					(*songs)++;

			}
			// Next file
			error = _findnext( handle, &finfo );
		}
	}

	_findclose( handle );
}

int Cmp3InfoData::size()
{
    return sizeof(Cmp3InfoData);
}
